@@ -1,54 +1,28 @@
 /*
+ * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @license AGPL-3.0
  *
- *    OPEN-XCHANGE legal information
+ * This code is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *    All intellectual property rights in the Software are protected by
- *    international copyright laws.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Affero General Public License for more details.
  *
+ * You should have received a copy of the GNU Affero General Public License
+ * along with OX App Suite.  If not, see <https://www.gnu.org/licenses/agpl-3.0.txt>.
  *
- *    In some countries OX, OX Open-Xchange, open xchange and OXtender
- *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the OX Software GmbH group of companies.
- *    The use of the Logos is not covered by the GNU General Public License.
- *    Instead, you are allowed to use these Logos according to the terms and
- *    conditions of the Creative Commons License, Version 2.5, Attribution,
- *    Non-commercial, ShareAlike, and the interpretation of the term
- *    Non-commercial applicable to the aforementioned license is published
- *    on the web site http://www.open-xchange.com/EN/legal/index.html.
- *
- *    Please make sure that third-party modules and libraries are used
- *    according to their respective licenses.
- *
- *    Any modifications to this package must retain all copyright notices
- *    of the original copyright holder(s) for the original code used.
- *
- *    After any such modifications, the original and derivative code shall remain
- *    under the copyright of the copyright holder(s) and/or original author(s)per
- *    the Attribution and Assignment Agreement that can be located at
- *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
- *    given Attribution for the derivative code and a license granting use.
- *
- *     Copyright (C) 2016-2020 OX Software GmbH
- *     Mail: info@open-xchange.com
- *
- *
- *     This program is free software; you can redistribute it and/or modify it
- *     under the terms of the GNU General Public License, Version 2 as published
- *     by the Free Software Foundation.
- *
- *     This program is distributed in the hope that it will be useful, but
- *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
- *     for more details.
- *
- *     You should have received a copy of the GNU General Public License along
- *     with this program; if not, write to the Free Software Foundation, Inc., 59
- *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Any use of the work other than as authorized under this license or copyright law is prohibited.
  *
  */
 
 package com.openexchange.sessionstorage.hazelcast;
 
+import static com.openexchange.java.Autoboxing.D;
+import static com.openexchange.java.Autoboxing.L;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
@@ -69,15 +43,18 @@ import java.util.concurrent.atomic.AtomicReference;
 import com.hazelcast.core.HazelcastException;
 import com.hazelcast.core.HazelcastInstance;
 import com.hazelcast.core.HazelcastInstanceNotActiveException;
-import com.hazelcast.core.IMap;
+import com.hazelcast.map.IMap;
 import com.hazelcast.query.Predicate;
-import com.hazelcast.query.SqlPredicate;
+import com.hazelcast.query.impl.predicates.SqlPredicate;
+import com.openexchange.config.ConfigTools;
+import com.openexchange.config.ConfigurationService;
 import com.openexchange.exception.OXException;
 import com.openexchange.server.ServiceExceptionCode;
 import com.openexchange.session.Session;
 import com.openexchange.session.SessionAttributes;
 import com.openexchange.sessionstorage.SessionStorageExceptionCodes;
 import com.openexchange.sessionstorage.SessionStorageService;
+import com.openexchange.sessionstorage.hazelcast.osgi.Services;
 import com.openexchange.sessionstorage.hazelcast.serialization.PortableSession;
 import com.openexchange.threadpool.ThreadPools;
 import io.micrometer.core.instrument.Gauge;
@@ -138,6 +115,8 @@ public class HazelcastSessionStorageService implements SessionStorageService {
     private final Unregisterer unregisterer;
     private final AtomicBoolean inactive;
     private final ConcurrentMap<String, AcquiredLatch> synchronizer;
+    private volatile Long maxShortTermIdleTime;
+    private volatile Long maxLongTermIdleTime;
 
     /**
      * Initializes a new {@link HazelcastSessionStorageService}.
@@ -263,7 +242,7 @@ public class HazelcastSessionStorageService implements SessionStorageService {
             if (timeoutMillis <= 0) {
                 storedSession = sessions().get(sessionId);
             } else {
-                Future<PortableSession> f = sessions().getAsync(sessionId);
+                Future<PortableSession> f = sessions().getAsync(sessionId).toCompletableFuture();
                 storedSession = getFrom(f, timeoutMillis);
                 if (null == storedSession && f.isCancelled()) {
                     LOG.warn("Session {} could not be retrieved from session storage within {}msec.", sessionId, Long.valueOf(timeoutMillis));
@@ -327,7 +306,7 @@ public class HazelcastSessionStorageService implements SessionStorageService {
         try {
             IMap<String, PortableSession> sessionsMap = sessions();
             for (Session session : sessions) {
-                sessionsMap.putIfAbsent(session.getSessionID(), new PortableSession(session));
+                sessionsMap.putIfAbsent(session.getSessionID(), new PortableSession(session), -1, TimeUnit.MILLISECONDS, getMaxIdleTime(session), TimeUnit.MILLISECONDS);
             }
         } catch (HazelcastInstanceNotActiveException e) {
             throw handleNotActiveException(e);
@@ -343,7 +322,7 @@ public class HazelcastSessionStorageService implements SessionStorageService {
         }
         ensureActive();
         try {
-            return null == sessions().putIfAbsent(session.getSessionID(), new PortableSession(session));
+            return null == sessions().putIfAbsent(session.getSessionID(), new PortableSession(session), -1, TimeUnit.MILLISECONDS, getMaxIdleTime(session), TimeUnit.MILLISECONDS);
         } catch (HazelcastInstanceNotActiveException e) {
             throw handleNotActiveException(e);
         } catch (HazelcastException e) {
@@ -361,7 +340,7 @@ public class HazelcastSessionStorageService implements SessionStorageService {
         if (null != session) {
             ensureActive();
             try {
-                sessions().set(session.getSessionID(), new PortableSession(session), 0, TimeUnit.SECONDS);
+                sessions().set(session.getSessionID(), new PortableSession(session), -1, TimeUnit.MILLISECONDS, getMaxIdleTime(session), TimeUnit.MILLISECONDS);
             } catch (HazelcastInstanceNotActiveException e) {
                 throw handleNotActiveException(e);
             } catch (HazelcastException e) {
@@ -429,7 +408,7 @@ public class HazelcastSessionStorageService implements SessionStorageService {
          */
         Map<String, Future<PortableSession>> futures = new HashMap<String, Future<PortableSession>>(size);
         for (String sessionID : sessionIds) {
-            futures.put(sessionID, sessions.removeAsync(sessionID));
+            futures.put(sessionID, sessions.removeAsync(sessionID).toCompletableFuture());
         }
         /*
          * collect removed sessions
@@ -602,6 +581,7 @@ public class HazelcastSessionStorageService implements SessionStorageService {
             return new ArrayList<Session>(sessions().values());
         } catch (HazelcastInstanceNotActiveException e) {
             inactive.set(true);
+            LOG.trace("", e);
         } catch (HazelcastException e) {
             LOG.debug("", e);
         } catch (OXException e) {
@@ -818,6 +798,7 @@ public class HazelcastSessionStorageService implements SessionStorageService {
      *
      * @param sessionIDs The session ID to touch
      * @return The number of successfully touched sessions
+     * @throws OXException In case of hazelcast error
      */
     public int touch(List<String> sessionIDs) throws OXException {
         int touched = 0;
@@ -852,7 +833,7 @@ public class HazelcastSessionStorageService implements SessionStorageService {
      * @return The first found matching session, or <code>null</code> if not found
      * @throws OXException
      */
-    private PortableSession findSession(Predicate<?, ?> predicate) throws OXException, HazelcastException {
+    private PortableSession findSession(Predicate<String, PortableSession> predicate) throws OXException, HazelcastException {
         IMap<String, PortableSession> sessions = sessions();
         if (null == sessions) {
             return null;
@@ -909,6 +890,68 @@ public class HazelcastSessionStorageService implements SessionStorageService {
         }
     }
 
+    /**
+     * Gets the default max idle time for sessions not to be put in long term containers
+     *
+     * @return The max idle time in milliseconds
+     */
+    private long getMaxShortTermIdleTime() {
+        if (null != maxShortTermIdleTime) {
+            return maxShortTermIdleTime.longValue();
+        }
+        int defaultValue = 3600000;
+        ConfigurationService configService = Services.getService(ConfigurationService.class);
+        if (null != configService) {
+            maxShortTermIdleTime = L(configService.getIntProperty("com.openexchange.sessiond.sessionDefaultLifeTime", defaultValue));
+            return maxShortTermIdleTime.longValue();
+        }
+        LOG.warn("ConfigurationService is not available, using default value: {}", L(defaultValue));
+        return defaultValue;
+    }
+
+    /**
+     * Gets the default max idle time for sessions to be put in long term containers
+     *
+     * @return The max idle time in milliseconds
+     */
+    private long getMaxLongTermIdleTime() {
+        if (null != maxLongTermIdleTime) {
+            return maxLongTermIdleTime.longValue();
+        }
+        String defaultValue = "1W";
+        ConfigurationService configService = Services.getService(ConfigurationService.class);
+        if (null != configService) {
+            String value = configService.getProperty("com.openexchange.sessiond.sessionLongLifeTime", defaultValue);
+            Long tmp = L(ConfigTools.parseTimespan(value));
+            if (-1L == tmp.longValue()) {
+                LOG.warn("Invalid value {}, using default value {}", value, defaultValue);
+                return ConfigTools.parseTimespan(defaultValue);
+            }
+            maxLongTermIdleTime = tmp;
+            return maxLongTermIdleTime.longValue();
+        }
+        LOG.warn("ConfigurationService is not available, using default value: {}", defaultValue);
+        return ConfigTools.parseTimespan(defaultValue);
+    }
+
+    /**
+     * Gets the default max idle time for given session
+     *
+     * @param session The session
+     * @return The max idle time in milliseconds
+     */
+    private long getMaxIdleTime(Session session) {
+        if (null != session) {
+            if (false == session.isStaySignedIn() || session.isTransient()) {
+                // Do not put in long term containers
+                return getMaxShortTermIdleTime();
+            }
+            // Put in long term containers
+            return getMaxLongTermIdleTime();
+        }
+        return 0;
+    }
+
     // ------------------------------------ Exception handling ------------------------------------------------------------ //
 
     private OXException handleRuntimeException(RuntimeException e, OXException optRegular) {
@@ -948,9 +991,9 @@ public class HazelcastSessionStorageService implements SessionStorageService {
     private void initMetrics() {
         Gauge.builder("appsuite.sessionstorage.sessions.total", () -> {
                 try {
-                    return sessions().getLocalMapStats().getOwnedEntryCount();
+                    return L(sessions().getLocalMapStats().getOwnedEntryCount());
                 } catch (OXException e) {
-                    return Double.NaN;
+                    return D(Double.NaN);
                 }
             })
             .description("Total number of stored sessions (owned or backups) held by this node.")
@@ -959,9 +1002,9 @@ public class HazelcastSessionStorageService implements SessionStorageService {
 
         Gauge.builder("appsuite.sessionstorage.memory.bytes", () -> {
                 try {
-                    return sessions().getLocalMapStats().getOwnedEntryMemoryCost();
+                    return L(sessions().getLocalMapStats().getOwnedEntryMemoryCost());
                 } catch (OXException e) {
-                    return Double.NaN;
+                    return D(Double.NaN);
                 }
             })
             .description("Consumed memory of stored sessions (owned or backups) held by this node in.")
@@ -970,9 +1013,9 @@ public class HazelcastSessionStorageService implements SessionStorageService {
 
         Gauge.builder("appsuite.sessionstorage.sessions.total", () -> {
                 try {
-                    return sessions().getLocalMapStats().getBackupEntryCount();
+                    return L(sessions().getLocalMapStats().getBackupEntryCount());
                 } catch (OXException e) {
-                    return Double.NaN;
+                    return D(Double.NaN);
                 }
             })
             .description("Total number of stored sessions (owned or backups) held by this node.")
@@ -981,9 +1024,9 @@ public class HazelcastSessionStorageService implements SessionStorageService {
 
         Gauge.builder("appsuite.sessionstorage.memory.bytes", () -> {
                 try {
-                    return sessions().getLocalMapStats().getBackupEntryMemoryCost();
+                    return L(sessions().getLocalMapStats().getBackupEntryMemoryCost());
                 } catch (OXException e) {
-                    return Double.NaN;
+                    return D(Double.NaN);
                 }
             })
             .description("Consumed memory of stored sessions (owned or backups) held by this node.")
